@@ -31,13 +31,12 @@
 
 /* clutter-actor.h */
 
+#include <glib-object.h>
 #include <pango/pango.h>
 #include <atk/atk.h>
 
-#include <clutter/clutter-types.h>
-
 #include <clutter/clutter-color.h>
-#include <clutter/clutter-effect.h>
+#include <clutter/clutter-types.h>
 #include <clutter/clutter-event.h>
 #include <clutter/clutter-shader.h>
 
@@ -277,6 +276,9 @@ struct _ClutterActorClass
   void (* apply_transform)      (ClutterActor           *actor,
                                  CoglMatrix             *matrix);
 
+  /* accessibility support */
+  AtkObject* (*get_accessible)  (ClutterActor *actor);
+
   /* event signals */
   gboolean (* event)                (ClutterActor         *actor,
                                      ClutterEvent         *event);
@@ -303,9 +305,6 @@ struct _ClutterActorClass
 
   void     (* queue_relayout)       (ClutterActor         *actor);
 
-  /* accessibility support */
-  AtkObject * (* get_accessible)    (ClutterActor         *actor);
-
   /*< private >*/
   /* padding for future expansion */
   gpointer _padding_dummy[30];
@@ -361,13 +360,6 @@ void                  clutter_actor_allocate_available_size   (ClutterActor
                                                                gfloat                 available_width,
                                                                gfloat                 available_height,
                                                                ClutterAllocationFlags flags);
-void                  clutter_actor_allocate_align_fill       (ClutterActor          *self,
-                                                               const ClutterActorBox *box,
-                                                               gdouble                x_align,
-                                                               gdouble                y_align,
-                                                               gboolean               x_fill,
-                                                               gboolean               y_fill,
-                                                               ClutterAllocationFlags flags);
 void                  clutter_actor_get_allocation_box        (ClutterActor          *self,
                                                                ClutterActorBox       *box);
 void                  clutter_actor_get_allocation_geometry   (ClutterActor          *self,
@@ -455,9 +447,6 @@ void                  clutter_actor_get_clip                  (ClutterActor
                                                                gfloat                *yoff,
                                                                gfloat                *width,
                                                                gfloat                *height);
-void                  clutter_actor_set_clip_to_allocation    (ClutterActor          *self,
-                                                               gboolean               clip_set);
-gboolean              clutter_actor_get_clip_to_allocation    (ClutterActor          *self);
 
 void                  clutter_actor_set_parent                (ClutterActor          *self,
                                                                ClutterActor          *parent);
@@ -465,8 +454,6 @@ ClutterActor *        clutter_actor_get_parent                (ClutterActor
 void                  clutter_actor_reparent                  (ClutterActor          *self,
                                                                ClutterActor          *new_parent);
 void                  clutter_actor_unparent                  (ClutterActor          *self);
-gboolean              clutter_actor_contains                  (ClutterActor          *self,
-							       ClutterActor          *descendant);
 ClutterActor*         clutter_actor_get_stage                 (ClutterActor          *actor);
 
 void                  clutter_actor_raise                     (ClutterActor          *self,
@@ -583,7 +570,7 @@ void                 clutter_actor_pop_internal       (ClutterActor         *sel
 
 gboolean             clutter_actor_has_allocation     (ClutterActor         *self);
 
-AtkObject *          clutter_actor_get_accessible     (ClutterActor         *self);
+AtkObject*           clutter_actor_get_accessible     (ClutterActor *actor);
 
 G_END_DECLS
 
