@@ -145,7 +145,6 @@
 #include "clutter-debug.h"
 #include "clutter-enum-types.h"
 #include "clutter-interval.h"
-#include "clutter-marshal.h"
 #include "clutter-private.h"
 #include "clutter-scriptable.h"
 #include "clutter-script-private.h"
@@ -159,13 +158,9 @@ enum
   PROP_DURATION,
   PROP_LOOP,
   PROP_TIMELINE,
-  PROP_ALPHA,
-
-  PROP_LAST
+  PROP_ALPHA
 };
 
-static GParamSpec *obj_props[PROP_LAST];
-
 enum
 {
   STARTED,
@@ -245,7 +240,9 @@ clutter_animation_real_completed (ClutterAnimation *self)
         p_value = clutter_interval_peek_initial_value (interval);
 
       if (animatable != NULL)
-        clutter_animatable_set_final_state (animatable, p_name, p_value);
+        clutter_animatable_set_final_state (animatable, self,
+                                            p_name,
+                                            p_value);
       else
         g_object_set_property (priv->object, p_name, p_value);
     }
@@ -460,11 +457,10 @@ clutter_animation_class_init (ClutterAnimationClass *klass)
    * Since: 1.0
    */
   pspec = g_param_spec_object ("object",
-                               P_("Object"),
-                               P_("Object to which the animation applies"),
+                               "Object",
+                               "Object to which the animation applies",
                                G_TYPE_OBJECT,
                                CLUTTER_PARAM_READWRITE);
-  obj_props[PROP_OBJECT] = pspec;
   g_object_class_install_property (gobject_class, PROP_OBJECT, pspec);
 
   /**
@@ -477,12 +473,11 @@ clutter_animation_class_init (ClutterAnimationClass *klass)
    * Since: 1.0
    */
   pspec = g_param_spec_ulong ("mode",
-                              P_("Mode"),
-                              P_("The mode of the animation"),
+                              "Mode",
+                              "The mode of the animation",
                               0, G_MAXULONG,
                               CLUTTER_LINEAR,
                               CLUTTER_PARAM_READWRITE);
-  obj_props[PROP_MODE] = pspec;
   g_object_class_install_property (gobject_class, PROP_MODE, pspec);
 
   /**
@@ -493,11 +488,10 @@ clutter_animation_class_init (ClutterAnimationClass *klass)
    * Since: 1.0
    */
   pspec = g_param_spec_uint ("duration",
-                             P_("Duration"),
-                             P_("Duration of the animation, in milliseconds"),
+                             "Duration",
+                             "Duration of the animation, in milliseconds",
                              0, G_MAXUINT, 0,
                              CLUTTER_PARAM_READWRITE);
-  obj_props[PROP_DURATION] = pspec;
   g_object_class_install_property (gobject_class, PROP_DURATION, pspec);
 
   /**
@@ -508,11 +502,10 @@ clutter_animation_class_init (ClutterAnimationClass *klass)
    * Since: 1.0
    */
   pspec = g_param_spec_boolean ("loop",
-                                P_("Loop"),
-                                P_("Whether the animation should loop"),
+                                "Loop",
+                                "Whether the animation should loop",
                                 FALSE,
                                 CLUTTER_PARAM_READWRITE);
-  obj_props[PROP_LOOP] = pspec;
   g_object_class_install_property (gobject_class, PROP_LOOP, pspec);
 
   /**
@@ -523,11 +516,10 @@ clutter_animation_class_init (ClutterAnimationClass *klass)
    * Since: 1.0
    */
   pspec = g_param_spec_object ("timeline",
-                               P_("Timeline"),
-                               P_("The timeline used by the animation"),
+                               "Timeline",
+                               "The timeline used by the animation",
                                CLUTTER_TYPE_TIMELINE,
                                CLUTTER_PARAM_READWRITE);
-  obj_props[PROP_TIMELINE] = pspec;
   g_object_class_install_property (gobject_class, PROP_TIMELINE, pspec);
 
   /**
@@ -538,11 +530,10 @@ clutter_animation_class_init (ClutterAnimationClass *klass)
    * Since: 1.0
    */
   pspec = g_param_spec_object ("alpha",
-                               P_("Alpha"),
-                               P_("The alpha used by the animation"),
+                               "Alpha",
+                               "The alpha used by the animation",
                                CLUTTER_TYPE_ALPHA,
                                CLUTTER_PARAM_READWRITE);
-  obj_props[PROP_ALPHA] = pspec;
   g_object_class_install_property (gobject_class, PROP_ALPHA, pspec);
 
   /**
@@ -560,7 +551,7 @@ clutter_animation_class_init (ClutterAnimationClass *klass)
                   G_SIGNAL_RUN_LAST,
                   G_STRUCT_OFFSET (ClutterAnimationClass, started),
                   NULL, NULL,
-                  _clutter_marshal_VOID__VOID,
+                  g_cclosure_marshal_VOID__VOID,
                   G_TYPE_NONE, 0);
 
   /**
@@ -581,7 +572,7 @@ clutter_animation_class_init (ClutterAnimationClass *klass)
                   G_SIGNAL_RUN_LAST,
                   G_STRUCT_OFFSET (ClutterAnimationClass, completed),
                   NULL, NULL,
-                  _clutter_marshal_VOID__VOID,
+                  g_cclosure_marshal_VOID__VOID,
                   G_TYPE_NONE, 0);
 }
 
@@ -670,7 +661,9 @@ clutter_animation_validate_bind (ClutterAnimation *animation,
     {
       ClutterAnimatable *animatable = CLUTTER_ANIMATABLE (priv->object);
 
-      pspec = clutter_animatable_find_property (animatable, property_name);
+      pspec = clutter_animatable_find_property (animatable,
+                                                animation,
+                                                property_name);
     }
   else
     {
@@ -795,6 +788,7 @@ clutter_animation_bind (ClutterAnimation *animation,
 
   if (CLUTTER_IS_ANIMATABLE (priv->object))
     clutter_animatable_get_initial_state (CLUTTER_ANIMATABLE (priv->object),
+                                          animation,
                                           property_name,
                                           &initial);
   else
@@ -906,7 +900,9 @@ clutter_animation_update_interval (ClutterAnimation *animation,
     {
       ClutterAnimatable *animatable = CLUTTER_ANIMATABLE (priv->object);
 
-      pspec = clutter_animatable_find_property (animatable, property_name);
+      pspec = clutter_animatable_find_property (animatable,
+                                                animation,
+                                                property_name);
     }
   else
     {
@@ -1104,7 +1100,9 @@ on_alpha_notify (GObject          *gobject,
       if (apply)
         {
           if (is_animatable)
-            clutter_animatable_set_final_state (animatable, p_name, &value);
+            clutter_animatable_set_final_state (animatable, animation,
+                                                p_name,
+                                                &value);
           else
             g_object_set_property (priv->object, p_name, &value);
         }
@@ -1138,7 +1136,7 @@ clutter_animation_get_alpha_internal (ClutterAnimation *animation)
 
       priv->alpha = g_object_ref_sink (alpha);
 
-      _clutter_notify_by_pspec (G_OBJECT (animation), obj_props[PROP_ALPHA]);
+      g_object_notify (G_OBJECT (animation), "alpha");
     }
 
   return priv->alpha;
@@ -1173,7 +1171,7 @@ clutter_animation_get_timeline_internal (ClutterAnimation *animation)
   /* the alpha owns the timeline now */
   g_object_unref (timeline);
 
-  _clutter_notify_by_pspec (G_OBJECT (animation), obj_props[PROP_TIMELINE]);
+  g_object_notify (G_OBJECT (animation), "timeline");
 
   return timeline;
 }
@@ -1238,7 +1236,7 @@ clutter_animation_set_object (ClutterAnimation *animation,
   if (object != NULL)
     priv->object = g_object_ref (object);
 
-  _clutter_notify_by_pspec (G_OBJECT (animation), obj_props[PROP_OBJECT]);
+  g_object_notify (G_OBJECT (animation), "object");
 }
 
 /**
@@ -1287,7 +1285,7 @@ clutter_animation_set_mode (ClutterAnimation *animation,
 
   clutter_alpha_set_mode (alpha, mode);
 
-  _clutter_notify_by_pspec (G_OBJECT (animation), obj_props[PROP_MODE]);
+  g_object_notify (G_OBJECT (animation), "mode");
 
   g_object_thaw_notify (G_OBJECT (animation));
 }
@@ -1343,7 +1341,7 @@ clutter_animation_set_duration (ClutterAnimation *animation,
   clutter_timeline_set_duration (timeline, msecs);
   clutter_timeline_rewind (timeline);
 
-  _clutter_notify_by_pspec (G_OBJECT (animation), obj_props[PROP_DURATION]);
+  g_object_notify (G_OBJECT (animation), "duration");
 
   g_object_thaw_notify (G_OBJECT (animation));
 }
@@ -1376,7 +1374,7 @@ clutter_animation_set_loop (ClutterAnimation *animation,
   timeline = clutter_animation_get_timeline_internal (animation);
   clutter_timeline_set_loop (timeline, loop);
 
-  _clutter_notify_by_pspec (G_OBJECT (animation), obj_props[PROP_LOOP]);
+  g_object_notify (G_OBJECT (animation), "loop");
 
   g_object_thaw_notify (G_OBJECT (animation));
 }
@@ -1469,9 +1467,9 @@ clutter_animation_set_timeline (ClutterAnimation *animation,
 
   alpha = clutter_animation_get_alpha_internal (animation);
   clutter_alpha_set_timeline (alpha, timeline);
-  _clutter_notify_by_pspec (G_OBJECT (animation), obj_props[PROP_TIMELINE]);
-  _clutter_notify_by_pspec (G_OBJECT (animation), obj_props[PROP_DURATION]);
-  _clutter_notify_by_pspec (G_OBJECT (animation), obj_props[PROP_LOOP]);
+  g_object_notify (G_OBJECT (animation), "timeline");
+  g_object_notify (G_OBJECT (animation), "duration");
+  g_object_notify (G_OBJECT (animation), "loop");
 
   if (timeline)
     {
@@ -1599,11 +1597,11 @@ clutter_animation_set_alpha (ClutterAnimation *animation,
 
 out:
   /* emit all relevant notifications */
-  _clutter_notify_by_pspec (G_OBJECT (animation), obj_props[PROP_MODE]);
-  _clutter_notify_by_pspec (G_OBJECT (animation), obj_props[PROP_DURATION]);
-  _clutter_notify_by_pspec (G_OBJECT (animation), obj_props[PROP_LOOP]);
-  _clutter_notify_by_pspec (G_OBJECT (animation), obj_props[PROP_ALPHA]);
-  _clutter_notify_by_pspec (G_OBJECT (animation), obj_props[PROP_TIMELINE]);
+  g_object_notify (G_OBJECT (animation), "mode");
+  g_object_notify (G_OBJECT (animation), "duration");
+  g_object_notify (G_OBJECT (animation), "loop");
+  g_object_notify (G_OBJECT (animation), "alpha");
+  g_object_notify (G_OBJECT (animation), "timeline");
 
   g_object_thaw_notify (G_OBJECT (animation));
 }
@@ -1748,6 +1746,7 @@ done:
 
       if (CLUTTER_IS_ANIMATABLE (priv->object))
         clutter_animatable_get_initial_state (CLUTTER_ANIMATABLE (priv->object),
+                                              animation,
                                               property_name,
                                               &cur_value);
       else
@@ -1773,6 +1772,7 @@ done:
     {
       if (CLUTTER_IS_ANIMATABLE (priv->object))
         clutter_animatable_set_final_state (CLUTTER_ANIMATABLE (priv->object),
+                                            animation,
                                             property_name,
                                             &real_value);
       else
@@ -1791,6 +1791,7 @@ clutter_animation_setupv (ClutterAnimation    *animation,
   ClutterAnimationPrivate *priv = animation->priv;
   ClutterAnimatable *animatable = NULL;
   GObjectClass *klass = NULL;
+  gboolean is_fixed = FALSE;
   gint i;
 
   if (CLUTTER_IS_ANIMATABLE (priv->object))
@@ -1802,7 +1803,6 @@ clutter_animation_setupv (ClutterAnimation    *animation,
     {
       const gchar *property_name = properties[i];
       GParamSpec *pspec;
-      gboolean is_fixed = FALSE;
 
       if (g_str_has_prefix (property_name, "fixed::"))
         {
@@ -1811,7 +1811,9 @@ clutter_animation_setupv (ClutterAnimation    *animation,
         }
 
       if (animatable != NULL)
-        pspec = clutter_animatable_find_property (animatable, property_name);
+        pspec = clutter_animatable_find_property (animatable,
+                                                  animation,
+                                                  property_name);
       else
         pspec = g_object_class_find_property (klass, property_name);
 
@@ -1911,6 +1913,7 @@ clutter_animation_setup_valist (ClutterAnimation *animation,
 
           if (animatable != NULL)
             pspec = clutter_animatable_find_property (animatable,
+                                                      animation,
                                                       property_name);
           else
             pspec = g_object_class_find_property (klass, property_name);
