@@ -78,13 +78,9 @@ enum
   PROP_DIRECTION,
   PROP_CENTER_X,
   PROP_CENTER_Y,
-  PROP_CENTER_Z,
-
-  PROP_LAST
+  PROP_CENTER_Z
 };
 
-static GParamSpec *obj_props[PROP_LAST];
-
 typedef struct {
   gdouble angle;
 } RotateFrameClosure;
@@ -274,12 +270,11 @@ clutter_behaviour_rotate_class_init (ClutterBehaviourRotateClass *klass)
    * Since: 0.4
    */
   pspec = g_param_spec_double ("angle-start",
-                               P_("Angle Begin"),
-                               P_("Initial angle"),
+                               "Angle Begin",
+                               "Initial angle",
                                0.0, 360.0,
                                0.0,
                                CLUTTER_PARAM_READWRITE);
-  obj_props[PROP_ANGLE_START] = pspec;
   g_object_class_install_property (gobject_class,
                                    PROP_ANGLE_START,
                                    pspec);
@@ -292,12 +287,11 @@ clutter_behaviour_rotate_class_init (ClutterBehaviourRotateClass *klass)
    * Since: 0.4
    */
   pspec = g_param_spec_double ("angle-end",
-                               P_("Angle End"),
-                               P_("Final angle"),
+                               "Angle End",
+                               "Final angle",
                                0.0, 360.0,
                                0.0,
                                CLUTTER_PARAM_READWRITE);
-  obj_props[PROP_ANGLE_END] = pspec;
   g_object_class_install_property (gobject_class,
                                    PROP_ANGLE_END,
                                    pspec);
@@ -310,12 +304,11 @@ clutter_behaviour_rotate_class_init (ClutterBehaviourRotateClass *klass)
    * Since: 0.4
    */
   pspec = g_param_spec_enum ("axis",
-                             P_("Axis"),
-                             P_("Axis of rotation"),
+                             "Axis",
+                             "Axis of rotation",
                              CLUTTER_TYPE_ROTATE_AXIS,
                              CLUTTER_Z_AXIS,
                              CLUTTER_PARAM_READWRITE);
-  obj_props[PROP_AXIS] = pspec;
   g_object_class_install_property (gobject_class,
                                    PROP_AXIS,
                                    pspec);
@@ -328,12 +321,11 @@ clutter_behaviour_rotate_class_init (ClutterBehaviourRotateClass *klass)
    * Since: 0.4
    */
   pspec = g_param_spec_enum ("direction",
-                             P_("Direction"),
-                             P_("Direction of rotation"),
+                             "Direction",
+                             "Direction of rotation",
                              CLUTTER_TYPE_ROTATE_DIRECTION,
                              CLUTTER_ROTATE_CW,
                              CLUTTER_PARAM_READWRITE);
-  obj_props[PROP_DIRECTION] = pspec;
   g_object_class_install_property (gobject_class,
                                    PROP_DIRECTION,
                                    pspec);
@@ -346,12 +338,11 @@ clutter_behaviour_rotate_class_init (ClutterBehaviourRotateClass *klass)
    * Since: 0.4
    */
   pspec = g_param_spec_int ("center-x",
-                            P_("Center X"),
-                            P_("X coordinate of the center of rotation"),
+                            "Center-X",
+                            "X center of rotation",
                             -G_MAXINT, G_MAXINT,
                             0,
                             CLUTTER_PARAM_READWRITE);
-  obj_props[PROP_CENTER_X] = pspec;
   g_object_class_install_property (gobject_class,
                                    PROP_CENTER_X,
                                    pspec);
@@ -364,12 +355,11 @@ clutter_behaviour_rotate_class_init (ClutterBehaviourRotateClass *klass)
    * Since: 0.4
    */
   pspec = g_param_spec_int ("center-y",
-                            P_("Center Y"),
-                            P_("Y coordinate of the center of rotation"),
+                            "Center-Y",
+                            "Y center of rotation",
                             -G_MAXINT, G_MAXINT,
                             0,
                             CLUTTER_PARAM_READWRITE);
-  obj_props[PROP_CENTER_Y] = pspec;
   g_object_class_install_property (gobject_class,
                                    PROP_CENTER_Y,
                                    pspec);
@@ -382,12 +372,11 @@ clutter_behaviour_rotate_class_init (ClutterBehaviourRotateClass *klass)
    * Since: 0.4
    */
   pspec = g_param_spec_int ("center-z",
-                            P_("Center Z"),
-                            P_("Z coordinate of the center of rotation"),
+                            "Center-Z",
+                            "Z center of rotation",
                             -G_MAXINT, G_MAXINT,
                             0,
                             CLUTTER_PARAM_READWRITE);
-  obj_props[PROP_CENTER_Z] = pspec;
   g_object_class_install_property (gobject_class,
                                    PROP_CENTER_Z,
                                    pspec);
@@ -486,7 +475,7 @@ clutter_behaviour_rotate_set_axis (ClutterBehaviourRotate *rotate,
     {
       priv->axis = axis;
 
-      _clutter_notify_by_pspec (G_OBJECT (rotate), obj_props[PROP_AXIS]);
+      g_object_notify (G_OBJECT (rotate), "axis");
     }
 }
 
@@ -532,7 +521,7 @@ clutter_behaviour_rotate_set_direction (ClutterBehaviourRotate *rotate,
     {
       priv->direction = direction;
 
-      _clutter_notify_by_pspec (G_OBJECT (rotate), obj_props[PROP_DIRECTION]);
+      g_object_notify (G_OBJECT (rotate), "direction");
     }
 }
 
@@ -592,14 +581,14 @@ clutter_behaviour_rotate_set_bounds (ClutterBehaviourRotate *rotate,
     {
       priv->angle_start = clamp_angle (angle_start);
 
-      _clutter_notify_by_pspec (G_OBJECT (rotate), obj_props[PROP_ANGLE_START]);
+      g_object_notify (G_OBJECT (rotate), "angle-start");
     }
 
   if (priv->angle_end != angle_end)
     {
       priv->angle_end = clamp_angle (angle_end);
 
-      _clutter_notify_by_pspec (G_OBJECT (rotate), obj_props[PROP_ANGLE_END]);
+      g_object_notify (G_OBJECT (rotate), "angle-end");
     }
 
   g_object_thaw_notify (G_OBJECT (rotate));
@@ -634,19 +623,19 @@ clutter_behaviour_rotate_set_center (ClutterBehaviourRotate *rotate,
   if (priv->center_x != x)
     {
       priv->center_x = x;
-      _clutter_notify_by_pspec (G_OBJECT (rotate), obj_props[PROP_CENTER_X]);
+      g_object_notify (G_OBJECT (rotate), "center-x");
     }
 
   if (priv->center_y != y)
     {
       priv->center_y = y;
-      _clutter_notify_by_pspec (G_OBJECT (rotate), obj_props[PROP_CENTER_Y]);
+      g_object_notify (G_OBJECT (rotate), "center-y");
     }
 
   if (priv->center_z != z)
     {
       priv->center_z = z;
-      _clutter_notify_by_pspec (G_OBJECT (rotate), obj_props[PROP_CENTER_Z]);
+      g_object_notify (G_OBJECT (rotate), "center-z");
     }
 
   g_object_thaw_notify (G_OBJECT (rotate));
