@@ -31,22 +31,33 @@
  * all the necessary logic for dragging actors.
  *
  * The simplest usage of #ClutterDragAction consists in adding it to
- * a #ClutterActor and setting it as reactive; for instance, the following
- * code:
+ * a #ClutterActor and connecting to the #ClutterDragAction::drag-motion
+ * signal handler to move the actor; for instance:
  *
  * |[
- *   clutter_actor_add_action (actor, clutter_drag_action_new ());
- *   clutter_actor_set_reactive (actor, TRUE);
+ *   ClutterAction *action = clutter_drag_action_new ();
+ *
+ *   g_signal_connect (action, "drag-motion",
+ *                     G_CALLBACK (on_drag_motion),
+ *                     NULL);
+ *
+ *   clutter_actor_add_action (actor, action);
  * ]|
  *
- * will automatically result in the actor moving to follow the pointer
- * whenever the pointer's button is pressed over the actor and moved
- * across the stage.
+ * Where the on_drag_motion() signal handler calls clutter_actor_move_by()
+ * using the delta between motion events passed to the handler:
  *
- * The #ClutterDragAction will signal the begin and the end of a dragging
- * through the #ClutterDragAction::drag-begin and #ClutterDragAction::drag-end
- * signals, respectively. Each pointer motion during a drag will also result
- * in the #ClutterDragAction::drag-motion signal to be emitted.
+ * |[
+ *   static void
+ *   on_drag_motion (ClutterDragAction   *action,
+ *                   ClutterActor        *actor,
+ *                   gfloat               delta_x,
+ *                   gfloat               delta_y,
+ *                   ClutterModifierType  modifiers)
+ *   {
+ *     clutter_actor_move_by (actor, delta_x, delta_y);
+ *   }
+ * ]|
  *
  * It is also possible to set another #ClutterActor as the dragged actor
  * by calling clutter_drag_action_set_drag_handle() from within a handle
@@ -72,8 +83,7 @@ struct _ClutterDragActionPrivate
 {
   ClutterActor *stage;
 
-  gfloat x_drag_threshold;
-  gfloat y_drag_threshold;
+  gfloat drag_threshold;
   ClutterActor *drag_handle;
   ClutterDragAxis drag_axis;
 
@@ -83,6 +93,7 @@ struct _ClutterDragActionPrivate
   gfloat press_x;
   gfloat press_y;
   ClutterModifierType press_state;
+  gint press_button;
 
   gfloat last_motion_x;
   gfloat last_motion_y;
@@ -90,25 +101,19 @@ struct _ClutterDragActionPrivate
   gfloat transformed_press_x;
   gfloat transformed_press_y;
 
-  guint emit_delayed_press    : 1;
-  guint in_drag               : 1;
-  guint motion_events_enabled : 1;
+  guint emit_delayed_press : 1;
+  guint in_drag            : 1;
 };
 
 enum
 {
   PROP_0,
 
-  PROP_X_DRAG_THRESHOLD,
-  PROP_Y_DRAG_THRESHOLD,
+  PROP_DRAG_THRESHOLD,
   PROP_DRAG_HANDLE,
-  PROP_DRAG_AXIS,
-
-  PROP_LAST
+  PROP_DRAG_AXIS
 };
 
-static GParamSpec *obj_props[PROP_LAST];
-
 enum
 {
   DRAG_BEGIN,
@@ -134,12 +139,10 @@ emit_drag_begin (ClutterDragAction *action,
 {
   ClutterDragActionPrivate *priv = action->priv;
 
-  priv->motion_events_enabled = clutter_get_motion_events_enabled ();
-  clutter_set_motion_events_enabled (FALSE);
-
   g_signal_emit (action, drag_signals[DRAG_BEGIN], 0,
                  actor,
                  priv->press_x, priv->press_y,
+                 priv->press_button,
                  priv->press_state);
 }
 
@@ -190,8 +193,8 @@ emit_drag_motion (ClutterDragAction *action,
 
   if (priv->emit_delayed_press)
     {
-      if (ABS (delta_x) >= priv->x_drag_threshold ||
-          ABS (delta_y) >= priv->y_drag_threshold)
+      if (ABS (delta_x) >= priv->drag_threshold ||
+          ABS (delta_y) >= priv->drag_threshold)
         {
           priv->emit_delayed_press = FALSE;
 
@@ -220,6 +223,7 @@ emit_drag_end (ClutterDragAction *action,
     g_signal_emit (action, drag_signals[DRAG_END], 0,
                    actor,
                    priv->last_motion_x, priv->last_motion_y,
+                   clutter_event_get_button (event),
                    clutter_event_get_state (event));
 
   /* disconnect the capture */
@@ -229,8 +233,6 @@ emit_drag_end (ClutterDragAction *action,
       priv->capture_id = 0;
     }
 
-  clutter_set_motion_events_enabled (priv->motion_events_enabled);
-
   priv->in_drag = FALSE;
 }
 
@@ -269,12 +271,6 @@ on_captured_event (ClutterActor      *stage,
         emit_drag_end (action, actor, event);
       break;
 
-    case CLUTTER_ENTER:
-    case CLUTTER_LEAVE:
-      if (priv->in_drag)
-        return TRUE;
-      break;
-
     default:
       break;
     }
@@ -292,13 +288,11 @@ on_button_press (ClutterActor      *actor,
   if (!clutter_actor_meta_get_enabled (CLUTTER_ACTOR_META (action)))
     return FALSE;
 
-  if (clutter_event_get_button (event) != 1)
-    return FALSE;
-
   if (priv->stage == NULL)
     priv->stage = clutter_actor_get_stage (actor);
 
   clutter_event_get_coords (event, &priv->press_x, &priv->press_y);
+  priv->press_button = clutter_event_get_button (event);
   priv->press_state = clutter_event_get_state (event);
 
   priv->last_motion_x = priv->press_x;
@@ -310,7 +304,7 @@ on_button_press (ClutterActor      *actor,
                                        &priv->transformed_press_x,
                                        &priv->transformed_press_y);
 
-  if (priv->x_drag_threshold == 0 || priv->y_drag_threshold == 0)
+  if (priv->drag_threshold == 0)
     emit_drag_begin (action, actor, event);
   else
     priv->emit_delayed_press = TRUE;
@@ -354,22 +348,6 @@ clutter_drag_action_set_actor (ClutterActorMeta *meta,
   CLUTTER_ACTOR_META_CLASS (clutter_drag_action_parent_class)->set_actor (meta, actor);
 }
 
-static void
-clutter_drag_action_real_drag_motion (ClutterDragAction *action,
-                                      ClutterActor      *actor,
-                                      gfloat             delta_x,
-                                      gfloat             delta_y)
-{
-  ClutterActor *drag_handle;
-
-  if (action->priv->drag_handle != NULL)
-    drag_handle = action->priv->drag_handle;
-  else
-    drag_handle = actor;
-
-  clutter_actor_move_by (drag_handle, delta_x, delta_y);
-}
-
 static void
 clutter_drag_action_set_property (GObject      *gobject,
                                   guint         prop_id,
@@ -380,14 +358,8 @@ clutter_drag_action_set_property (GObject      *gobject,
 
   switch (prop_id)
     {
-    case PROP_X_DRAG_THRESHOLD:
-      clutter_drag_action_set_drag_threshold (action, g_value_get_uint (value),
-                                              action->priv->y_drag_threshold);
-      break;
-
-    case PROP_Y_DRAG_THRESHOLD:
-      clutter_drag_action_set_drag_threshold (action, action->priv->x_drag_threshold,
-                                              g_value_get_uint (value));
+    case PROP_DRAG_THRESHOLD:
+      clutter_drag_action_set_drag_threshold (action, g_value_get_uint (value));
       break;
 
     case PROP_DRAG_HANDLE:
@@ -413,12 +385,8 @@ clutter_drag_action_get_property (GObject    *gobject,
 
   switch (prop_id)
     {
-    case PROP_X_DRAG_THRESHOLD:
-      g_value_set_uint (value, priv->x_drag_threshold);
-      break;
-
-    case PROP_Y_DRAG_THRESHOLD:
-      g_value_set_uint (value, priv->y_drag_threshold);
+    case PROP_DRAG_THRESHOLD:
+      g_value_set_uint (value, priv->drag_threshold);
       break;
 
     case PROP_DRAG_HANDLE:
@@ -475,49 +443,25 @@ clutter_drag_action_class_init (ClutterDragActionClass *klass)
 
   meta_class->set_actor = clutter_drag_action_set_actor;
 
-  klass->drag_motion = clutter_drag_action_real_drag_motion;
-
   /**
-   * ClutterDragAction:x-drag-threshold:
+   * ClutterDragAction:drag-threshold:
    *
-   * The horizontal threshold, in pixels, that begins a drag action
+   * The threshold, in pixels, that begins a drag action
    *
    * When set to a non-zero value, #ClutterDragAction will only emit
-   * #ClutterDragAction::drag-begin if the pointer has moved
-   * horizontally at least of the given amount of pixels since
-   * the button press event
+   * #ClutterDragAction::drag-begin if the pointer has moved at least
+   * of the given amount of pixels since the button press event
    *
    * Since: 1.4
    */
-  pspec = g_param_spec_uint ("x-drag-threshold",
-                             P_("Horizontal Drag Threshold"),
-                             P_("The horizontal amount of pixels required to start dragging"),
+  pspec = g_param_spec_uint ("drag-threshold",
+                             "Drag Threshold",
+                             "The amount of pixels required to start "
+                             "dragging",
                              0, G_MAXUINT,
                              0,
                              CLUTTER_PARAM_READWRITE);
-  obj_props[PROP_X_DRAG_THRESHOLD] = pspec;
-  g_object_class_install_property (gobject_class, PROP_X_DRAG_THRESHOLD, pspec);
-
-  /**
-   * ClutterDragAction:y-drag-threshold:
-   *
-   * The vertical threshold, in pixels, that begins a drag action
-   *
-   * When set to a non-zero value, #ClutterDragAction will only emit
-   * #ClutterDragAction::drag-begin if the pointer has moved
-   * vertically at least of the given amount of pixels since
-   * the button press event
-   *
-   * Since: 1.4
-   */
-  pspec = g_param_spec_uint ("y-drag-threshold",
-                             P_("Vertical Drag Threshold"),
-                             P_("The vertical amount of pixels required to start dragging"),
-                             0, G_MAXUINT,
-                             0,
-                             CLUTTER_PARAM_READWRITE);
-  obj_props[PROP_Y_DRAG_THRESHOLD] = pspec;
-  g_object_class_install_property (gobject_class, PROP_Y_DRAG_THRESHOLD, pspec);
+  g_object_class_install_property (gobject_class, PROP_DRAG_THRESHOLD, pspec);
 
   /**
    * ClutterDragAction:drag-handle:
@@ -534,11 +478,10 @@ clutter_drag_action_class_init (ClutterDragActionClass *klass)
    * Since: 1.4
    */
   pspec = g_param_spec_object ("drag-handle",
-                               P_("Drag Handle"),
-                               P_("The actor that is being dragged"),
+                               "Drag Handle",
+                               "The actor that is being dragged",
                                CLUTTER_TYPE_ACTOR,
                                CLUTTER_PARAM_READWRITE);
-  obj_props[PROP_DRAG_HANDLE] = pspec;
   g_object_class_install_property (gobject_class, PROP_DRAG_HANDLE, pspec);
 
   /**
@@ -549,12 +492,11 @@ clutter_drag_action_class_init (ClutterDragActionClass *klass)
    * Since: 1.4
    */
   pspec = g_param_spec_enum ("drag-axis",
-                             P_("Drag Axis"),
-                             P_("Constraints the dragging to an axis"),
+                             "Drag Axis",
+                             "Constraints the dragging to an axis",
                              CLUTTER_TYPE_DRAG_AXIS,
                              CLUTTER_DRAG_AXIS_NONE,
                              CLUTTER_PARAM_READWRITE);
-  obj_props[PROP_DRAG_AXIS] = pspec;
   g_object_class_install_property (gobject_class, PROP_DRAG_AXIS, pspec);
 
   /**
@@ -563,14 +505,14 @@ clutter_drag_action_class_init (ClutterDragActionClass *klass)
    * @actor: the #ClutterActor attached to the action
    * @event_x: the X coordinate (in stage space) of the press event
    * @event_y: the Y coordinate (in stage space) of the press event
+   * @button: the button of the press event
    * @modifiers: the modifiers of the press event
    *
    * The ::drag-begin signal is emitted when the #ClutterDragAction
    * starts the dragging
    *
    * The emission of this signal can be delayed by using the
-   * #ClutterDragAction:x-drag-threshold and
-   * #ClutterDragAction:y-drag-threshold properties
+   * #ClutterDragAction:drag-threshold property
    *
    * Since: 1.4
    */
@@ -580,11 +522,12 @@ clutter_drag_action_class_init (ClutterDragActionClass *klass)
                   G_SIGNAL_RUN_LAST,
                   G_STRUCT_OFFSET (ClutterDragActionClass, drag_begin),
                   NULL, NULL,
-                  _clutter_marshal_VOID__OBJECT_FLOAT_FLOAT_FLAGS,
-                  G_TYPE_NONE, 4,
+                  clutter_marshal_VOID__OBJECT_FLOAT_FLOAT_INT_FLAGS,
+                  G_TYPE_NONE, 5,
                   CLUTTER_TYPE_ACTOR,
                   G_TYPE_FLOAT,
                   G_TYPE_FLOAT,
+                  G_TYPE_INT,
                   CLUTTER_TYPE_MODIFIER_TYPE);
 
   /**
@@ -597,6 +540,7 @@ clutter_drag_action_class_init (ClutterDragActionClass *klass)
    * @delta_y: the Y component of the distance between the press event
    *   that began the dragging and the current position of the pointer,
    *   as of the latest motion event
+   * @modifiers: the modifiers of the latest motion event
    *
    * The ::drag-motion signal is emitted for each motion event after
    * the #ClutterDragAction::drag-begin signal has been emitted.
@@ -607,13 +551,6 @@ clutter_drag_action_class_init (ClutterDragActionClass *klass)
    * stage coordinates of the latest motion event you can use
    * clutter_drag_action_get_motion_coords().
    *
-   * The default handler of the signal will call clutter_actor_move_by()
-   * either on @actor or, if set, of #ClutterDragAction:drag-handle using
-   * the @delta_x and @delta_y components of the dragging motion. If you
-   * want to override the default behaviour, you can connect to this
-   * signal and call g_signal_stop_emission_by_name() from within your
-   * callback.
-   *
    * Since: 1.4
    */
   drag_signals[DRAG_MOTION] =
@@ -622,7 +559,7 @@ clutter_drag_action_class_init (ClutterDragActionClass *klass)
                   G_SIGNAL_RUN_LAST,
                   G_STRUCT_OFFSET (ClutterDragActionClass, drag_motion),
                   NULL, NULL,
-                  _clutter_marshal_VOID__OBJECT_FLOAT_FLOAT,
+                  clutter_marshal_VOID__OBJECT_FLOAT_FLOAT,
                   G_TYPE_NONE, 3,
                   CLUTTER_TYPE_ACTOR,
                   G_TYPE_FLOAT,
@@ -634,6 +571,7 @@ clutter_drag_action_class_init (ClutterDragActionClass *klass)
    * @actor: the #ClutterActor attached to the action
    * @event_x: the X coordinate (in stage space) of the release event
    * @event_y: the Y coordinate (in stage space) of the release event
+   * @button: the button of the release event
    * @modifiers: the modifiers of the release event
    *
    * The ::drag-end signal is emitted at the end of the dragging,
@@ -650,11 +588,12 @@ clutter_drag_action_class_init (ClutterDragActionClass *klass)
                   G_SIGNAL_RUN_LAST,
                   G_STRUCT_OFFSET (ClutterDragActionClass, drag_end),
                   NULL, NULL,
-                  _clutter_marshal_VOID__OBJECT_FLOAT_FLOAT_FLAGS,
-                  G_TYPE_NONE, 4,
+                  clutter_marshal_VOID__OBJECT_FLOAT_FLOAT_INT_FLAGS,
+                  G_TYPE_NONE, 5,
                   CLUTTER_TYPE_ACTOR,
                   G_TYPE_FLOAT,
                   G_TYPE_FLOAT,
+                  G_TYPE_INT,
                   CLUTTER_TYPE_MODIFIER_TYPE);
 }
 
@@ -685,15 +624,14 @@ clutter_drag_action_new (void)
  * @action: a #ClutterDragAction
  * @threshold: a distance, in pixels
  *
- * Sets the horizontal and vertical drag thresholds that must be
- * cleared by the pointer before @action can begin the dragging
+ * Sets the drag threshold that must be cleared by the pointer
+ * before @action can begin the dragging
  *
  * Since: 1.4
  */
 void
 clutter_drag_action_set_drag_threshold (ClutterDragAction *action,
-                                        guint              x_threshold,
-					guint              y_threshold)
+                                        guint              threshold)
 {
   ClutterDragActionPrivate *priv;
 
@@ -701,45 +639,30 @@ clutter_drag_action_set_drag_threshold (ClutterDragAction *action,
 
   priv = action->priv;
 
-  if (priv->x_drag_threshold != x_threshold)
-    {
-      priv->x_drag_threshold = x_threshold;
+  if (priv->drag_threshold == threshold)
+    return;
 
-      _clutter_notify_by_pspec (G_OBJECT (action), obj_props[PROP_X_DRAG_THRESHOLD]);
-    }
+  priv->drag_threshold = threshold;
 
-  if (priv->y_drag_threshold != y_threshold)
-    {
-      priv->y_drag_threshold = y_threshold;
-
-      _clutter_notify_by_pspec (G_OBJECT (action), obj_props[PROP_Y_DRAG_THRESHOLD]);
-    }  
+  g_object_notify (G_OBJECT (action), "drag-threshold");
 }
 
 /**
  * clutter_drag_action_get_drag_threshold:
  * @action: a #ClutterDragAction
- * @x_threshold: (out): return location for the horizontal drag
- *   threshold value, in pixels
- * @y_threshold: (out): return location for the vertical drag
- *   threshold value, in pixels
  *
- * Retrieves the values set by clutter_drag_action_set_drag_threshold()
+ * Retrieves the value set by clutter_drag_action_set_drag_threshold()
+ *
+ * Return value: the drag threshold value, in pixels
  *
  * Since: 1.4
  */
-void
-clutter_drag_action_get_drag_threshold (ClutterDragAction *action,
-                                        guint             *x_threshold,
-					guint             *y_threshold)
+guint
+clutter_drag_action_get_drag_threshold (ClutterDragAction *action)
 {
-  g_return_if_fail (CLUTTER_IS_DRAG_ACTION (action));
+  g_return_val_if_fail (CLUTTER_IS_DRAG_ACTION (action), 0);
 
-  if (x_threshold)
-    *x_threshold = action->priv->x_drag_threshold;
-
-  if (y_threshold)
-    *y_threshold = action->priv->y_drag_threshold;    
+  return action->priv->drag_threshold;
 }
 
 /**
@@ -767,7 +690,7 @@ clutter_drag_action_set_drag_handle (ClutterDragAction *action,
 
   priv->drag_handle = handle;
 
-  _clutter_notify_by_pspec (G_OBJECT (action), obj_props[PROP_DRAG_HANDLE]);
+  g_object_notify (G_OBJECT (action), "drag-handle");
 }
 
 /**
@@ -815,7 +738,7 @@ clutter_drag_action_set_drag_axis (ClutterDragAction *action,
 
   priv->drag_axis = axis;
 
-  _clutter_notify_by_pspec (G_OBJECT (action), obj_props[PROP_DRAG_AXIS]);
+  g_object_notify (G_OBJECT (action), "drag-axis");
 }
 
 /**
