@@ -40,12 +40,18 @@ G_BEGIN_DECLS
 
 /* Forward delarations to avoid header catch 22's */
 typedef struct _ClutterActor            ClutterActor;
+
 typedef struct _ClutterStage            ClutterStage;
 typedef struct _ClutterContainer        ClutterContainer; /* dummy */
 typedef struct _ClutterChildMeta        ClutterChildMeta;
 typedef struct _ClutterLayoutMeta       ClutterLayoutMeta;
+typedef struct _ClutterActorMeta        ClutterActorMeta;
+
 typedef struct _ClutterAnimator         ClutterAnimator;
 
+typedef struct _ClutterAction           ClutterAction;
+typedef struct _ClutterConstraint       ClutterConstraint;
+
 typedef union _ClutterEvent             ClutterEvent;
 
 /**
@@ -182,6 +188,10 @@ struct _ClutterGeometry
 
 GType clutter_geometry_get_type (void) G_GNUC_CONST;
 
+void  clutter_geometry_union (const ClutterGeometry *geometry_a,
+                              const ClutterGeometry *geometry_b,
+                              ClutterGeometry       *result);
+
 /**
  * ClutterKnot:
  * @x: X coordinate of the knot
