@@ -97,10 +97,11 @@ cogl_gl_error_to_string (GLenum error_code);
 
 #endif /* COGL_GL_DEBUG */
 
-#define COGL_ENABLE_ALPHA_TEST        (1<<1)
-#define COGL_ENABLE_VERTEX_ARRAY      (1<<2)
-#define COGL_ENABLE_COLOR_ARRAY       (1<<3)
-#define COGL_ENABLE_BACKFACE_CULLING  (1<<4)
+#define COGL_ENABLE_BLEND             (1<<1)
+#define COGL_ENABLE_ALPHA_TEST        (1<<2)
+#define COGL_ENABLE_VERTEX_ARRAY      (1<<3)
+#define COGL_ENABLE_COLOR_ARRAY       (1<<4)
+#define COGL_ENABLE_BACKFACE_CULLING  (1<<5)
 
 void
 _cogl_features_init (void);
@@ -114,22 +115,29 @@ _cogl_enable (unsigned long flags);
 unsigned long
 _cogl_get_enable (void);
 
+typedef struct _CoglTextureUnit
+{
+  int              index;
+  CoglMatrixStack *matrix_stack;
+} CoglTextureUnit;
+
+CoglTextureUnit *
+_cogl_get_texture_unit (int index_);
+
+void
+_cogl_destroy_texture_units (void);
+
+unsigned int
+_cogl_get_max_texture_image_units (void);
+
 void
 _cogl_flush_face_winding (void);
 
 /* Disables the texcoord arrays that don't have a corresponding bit
-   set in the mask and sets ctx->texcoord_arrays_enabled to mask. Note
-   that it doesn't enable any extra texcoord arrays */
+   set in the mask */
 void
 _cogl_disable_other_texcoord_arrays (const CoglBitmask *mask);
 
-void
-_cogl_transform_point (const CoglMatrix *matrix_mv,
-                       const CoglMatrix *matrix_p,
-                       const float *viewport,
-                       float *x,
-                       float *y);
-
 #ifdef COGL_HAS_XLIB_SUPPORT
 
 /*
@@ -145,17 +153,6 @@ typedef enum _CoglXlibFilterReturn {
   COGL_XLIB_FILTER_REMOVE
 } CoglXlibFilterReturn;
 
-/*
- * CoglXlibFilterFunc:
- *
- * A callback function that can be registered with
- * _cogl_xlib_add_filter. The function should return
- * %COGL_XLIB_FILTER_REMOVE if it wants to prevent further processing
- * or %COGL_XLIB_FILTER_CONTINUE otherwise.
- */
-typedef CoglXlibFilterReturn (* CoglXlibFilterFunc) (XEvent *xevent,
-                                                     gpointer data);
-
 /*
  * cogl_xlib_handle_event:
  * @xevent: pointer to XEvent structure
@@ -174,55 +171,6 @@ typedef CoglXlibFilterReturn (* CoglXlibFilterFunc) (XEvent *xevent,
 CoglXlibFilterReturn
 _cogl_xlib_handle_event (XEvent *xevent);
 
-/*
- * _cogl_xlib_get_display:
- *
- * Return value: the Xlib display that will be used by the Xlib winsys
- * backend. The display needs to be set with _cogl_xlib_set_display()
- * before this function is called.
- */
-Display *
-_cogl_xlib_get_display (void);
-
-/*
- * cogl_xlib_set_display:
- *
- * Sets the Xlib display that Cogl will use for the Xlib winsys
- * backend. This function should eventually go away when Cogl gains a
- * more complete winsys abstraction.
- */
-void
-_cogl_xlib_set_display (Display *display);
-
-/*
- * _cogl_xlib_add_filter:
- *
- * Adds a callback function that will receive all X11 events. The
- * function can stop further processing of the event by return
- * %COGL_XLIB_FILTER_REMOVE.
- */
-void
-_cogl_xlib_add_filter (CoglXlibFilterFunc func,
-                       gpointer data);
-
-/*
- * _cogl_xlib_remove_filter:
- *
- * Removes a callback that was previously added with
- * _cogl_xlib_add_filter().
- */
-void
-_cogl_xlib_remove_filter (CoglXlibFilterFunc func,
-                          gpointer data);
-
 #endif /* COGL_HAS_XLIB_SUPPORT */
 
-typedef enum _CoglFeatureFlagsPrivate
-{
-  COGL_FEATURE_PRIVATE_ARB_FP = (1 << 0)
-} CoglFeatureFlagsPrivate;
-
-gboolean
-_cogl_features_available_private (CoglFeatureFlagsPrivate features);
-
 #endif /* __COGL_INTERNAL_H */
