@@ -16,9 +16,9 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * License along with this library. If not, see <http://www.gnu.org/licenses/>.
+ *
+ *
  */
 
 #ifdef HAVE_CONFIG_H
@@ -88,7 +88,7 @@ typedef CoglVertexBufferIndices  CoglJournalIndices;
 
 typedef struct _CoglJournalFlushState
 {
-  size_t              stride;
+  gsize              stride;
   /* Note: this is a pointer to handle fallbacks. It normally holds a VBO
    * offset, but when the driver doesn't support VBOs then this points into
    * our GArray of logged vertices. */
@@ -96,7 +96,7 @@ typedef struct _CoglJournalFlushState
   GLuint              vertex_offset;
 #ifndef HAVE_COGL_GL
   CoglJournalIndices *indices;
-  size_t              indices_type_size;
+  gsize              indices_type_size;
 #endif
   CoglMatrixStack    *modelview_stack;
 } CoglJournalFlushState;
@@ -110,7 +110,7 @@ typedef gboolean (*CoglJournalBatchTest) (CoglJournalEntry *entry0,
 void
 _cogl_journal_dump_quad_vertices (guint8 *data, int n_layers)
 {
-  size_t stride = GET_JOURNAL_VB_STRIDE_FOR_N_LAYERS (n_layers);
+  gsize stride = GET_JOURNAL_VB_STRIDE_FOR_N_LAYERS (n_layers);
   int i;
 
   _COGL_GET_CONTEXT (ctx, NO_RETVAL);
@@ -126,7 +126,8 @@ _cogl_journal_dump_quad_vertices (guint8 *data, int n_layers)
       guint8 *c = data + (POS_STRIDE * 4) + (i * stride * 4);
       int j;
 
-      if (cogl_debug_flags & COGL_DEBUG_DISABLE_SOFTWARE_TRANSFORM)
+      if (G_UNLIKELY (cogl_debug_flags &
+                      COGL_DEBUG_DISABLE_SOFTWARE_TRANSFORM))
         g_print ("v%d: x = %f, y = %f, rgba=0x%02X%02X%02X%02X",
                  i, v[0], v[1], c[0], c[1], c[2], c[3]);
       else
@@ -144,7 +145,7 @@ _cogl_journal_dump_quad_vertices (guint8 *data, int n_layers)
 void
 _cogl_journal_dump_quad_batch (guint8 *data, int n_layers, int n_quads)
 {
-  size_t byte_stride = GET_JOURNAL_VB_STRIDE_FOR_N_LAYERS (n_layers) * 4;
+  gsize byte_stride = GET_JOURNAL_VB_STRIDE_FOR_N_LAYERS (n_layers) * 4;
   int i;
 
   g_print ("_cogl_journal_dump_quad_batch: n_layers = %d, n_quads = %d\n",
@@ -234,7 +235,7 @@ _cogl_journal_flush_modelview_and_entries (CoglJournalEntry *batch_start,
    * issues, visually seeing what is batched and debugging blending
    * issues, plus it looks quite cool.
    */
-  if (cogl_debug_flags & COGL_DEBUG_RECTANGLES)
+  if (G_UNLIKELY (cogl_debug_flags & COGL_DEBUG_RECTANGLES))
     {
       static CoglHandle outline = COGL_INVALID_HANDLE;
       guint8 color_intensity;
@@ -262,7 +263,7 @@ _cogl_journal_flush_modelview_and_entries (CoglJournalEntry *batch_start,
                                   color_intensity : 0,
                                   0xff);
       _cogl_material_flush_gl_state (outline, NULL);
-      cogl_enable (COGL_ENABLE_VERTEX_ARRAY);
+      _cogl_enable (COGL_ENABLE_VERTEX_ARRAY);
       for (i = 0; i < batch_len; i++)
         GE( glDrawArrays (GL_LINE_LOOP, 4 * i + state->vertex_offset, 4) );
 
@@ -304,10 +305,10 @@ compare_entry_modelviews (CoglJournalEntry *entry0,
  * materials, but they may not all have the same modelview matrix */
 static void
 _cogl_journal_flush_material_and_entries (CoglJournalEntry *batch_start,
-                                          gint              batch_len,
+                                          int               batch_len,
                                           void             *data)
 {
-  gulong                 enable_flags = 0;
+  unsigned long enable_flags = 0;
 
   _COGL_GET_CONTEXT (ctx, NO_RETVAL);
 
@@ -318,7 +319,7 @@ _cogl_journal_flush_material_and_entries (CoglJournalEntry *batch_start,
                                  &batch_start->flush_options);
 
   /* FIXME: This api is a bit yukky, ideally it will be removed if we
-   * re-work the cogl_enable mechanism */
+   * re-work the _cogl_enable mechanism */
   enable_flags |= _cogl_material_get_cogl_enable_flags (batch_start->material);
 
   if (ctx->enable_backface_culling)
@@ -326,12 +327,12 @@ _cogl_journal_flush_material_and_entries (CoglJournalEntry *batch_start,
 
   enable_flags |= COGL_ENABLE_VERTEX_ARRAY;
   enable_flags |= COGL_ENABLE_COLOR_ARRAY;
-  cogl_enable (enable_flags);
+  _cogl_enable (enable_flags);
   _cogl_flush_face_winding ();
 
   /* If we haven't transformed the quads in software then we need to also break
    * up batches according to changes in the modelview matrix... */
-  if (cogl_debug_flags & COGL_DEBUG_DISABLE_SOFTWARE_TRANSFORM)
+  if (G_UNLIKELY (cogl_debug_flags & COGL_DEBUG_DISABLE_SOFTWARE_TRANSFORM))
     {
       batch_and_call (batch_start,
                       batch_len,
@@ -367,11 +368,10 @@ compare_entry_materials (CoglJournalEntry *entry0, CoglJournalEntry *entry1)
 static void
 _cogl_journal_flush_texcoord_vbo_offsets_and_entries (
                                           CoglJournalEntry *batch_start,
-                                          gint              batch_len,
+                                          int               batch_len,
                                           void             *data)
 {
   CoglJournalFlushState *state = data;
-  int                    prev_n_texcoord_arrays_enabled;
   int                    i;
 
   _COGL_GET_CONTEXT (ctx, NO_RETVAL);
@@ -394,14 +394,10 @@ _cogl_journal_flush_texcoord_vbo_offsets_and_entries (
                                       (POS_STRIDE + COLOR_STRIDE) * 4 +
                                       TEX_STRIDE * 4 * i)));
     }
-  prev_n_texcoord_arrays_enabled =
-    ctx->n_texcoord_arrays_enabled;
-  ctx->n_texcoord_arrays_enabled = batch_start->n_layers;
-  for (; i < prev_n_texcoord_arrays_enabled; i++)
-    {
-      GE (glClientActiveTexture (GL_TEXTURE0 + i));
-      GE (glDisableClientState (GL_TEXTURE_COORD_ARRAY));
-    }
+
+  _cogl_bitmask_clear_all (&ctx->temp_bitmask);
+  _cogl_bitmask_set_range (&ctx->temp_bitmask, batch_start->n_layers, TRUE);
+  _cogl_disable_other_texcoord_arrays (&ctx->temp_bitmask);
 
   batch_and_call (batch_start,
                   batch_len,
@@ -423,11 +419,11 @@ compare_entry_n_layers (CoglJournalEntry *entry0, CoglJournalEntry *entry1)
  * of journal entries */
 static void
 _cogl_journal_flush_vbo_offsets_and_entries (CoglJournalEntry *batch_start,
-                                             gint              batch_len,
+                                             int               batch_len,
                                              void             *data)
 {
   CoglJournalFlushState   *state = data;
-  size_t                   stride;
+  gsize                   stride;
 #ifndef HAVE_COGL_GL
   int                      needed_indices = batch_len * 6;
   CoglHandle               indices_handle;
@@ -479,7 +475,7 @@ _cogl_journal_flush_vbo_offsets_and_entries (CoglJournalEntry *batch_start,
    * VBO using a vertex offset passed to glDraw{Arrays,Elements} */
   state->vertex_offset = 0;
 
-  if (cogl_debug_flags & COGL_DEBUG_JOURNAL)
+  if (G_UNLIKELY (cogl_debug_flags & COGL_DEBUG_JOURNAL))
     {
       guint8 *verts;
 
@@ -502,7 +498,7 @@ _cogl_journal_flush_vbo_offsets_and_entries (CoglJournalEntry *batch_start,
   /* progress forward through the VBO containing all our vertices */
   state->vbo_offset += (stride * 4 * batch_len);
   if (G_UNLIKELY (cogl_debug_flags & COGL_DEBUG_JOURNAL))
-    g_print ("new vbo offset = %lu\n", (gulong)state->vbo_offset);
+    g_print ("new vbo offset = %lu\n", (unsigned long)state->vbo_offset);
 }
 
 static gboolean
@@ -524,7 +520,7 @@ compare_entry_strides (CoglJournalEntry *entry0, CoglJournalEntry *entry1)
 static GLuint
 upload_vertices_to_vbo (GArray *vertices, CoglJournalFlushState *state)
 {
-  size_t needed_vbo_len;
+  gsize needed_vbo_len;
   GLuint journal_vbo;
 
   _COGL_GET_CONTEXT (ctx, 0);
@@ -657,19 +653,18 @@ _cogl_journal_init (void)
 }
 
 void
-_cogl_journal_log_quad (float         x_1,
-                        float         y_1,
-                        float         x_2,
-                        float         y_2,
+_cogl_journal_log_quad (const float  *position,
                         CoglHandle    material,
                         int           n_layers,
                         guint32       fallback_layers,
                         GLuint        layer0_override_texture,
+                        const CoglMaterialWrapModeOverrides *
+                                      wrap_mode_overrides,
                         const float  *tex_coords,
                         unsigned int  tex_coords_len)
 {
-  size_t            stride;
-  size_t            byte_stride;
+  gsize            stride;
+  gsize            byte_stride;
   int               next_vert;
   GLfloat          *v;
   GLubyte          *c;
@@ -722,15 +717,20 @@ _cogl_journal_log_quad (float         x_1,
       memcpy (c, src_c, 4);
     }
 
+#define X0 0
+#define Y0 1
+#define X1 2
+#define Y1 3
+
   if (G_UNLIKELY (cogl_debug_flags & COGL_DEBUG_DISABLE_SOFTWARE_TRANSFORM))
     {
-      v[0] = x_1; v[1] = y_1;
+      v[0] = position[X0]; v[1] = position[Y0];
       v += stride;
-      v[0] = x_1; v[1] = y_2;
+      v[0] = position[X0]; v[1] = position[Y1];
       v += stride;
-      v[0] = x_2; v[1] = y_2;
+      v[0] = position[X1]; v[1] = position[Y1];
       v += stride;
-      v[0] = x_2; v[1] = y_1;
+      v[0] = position[X1]; v[1] = position[Y0];
     }
   else
     {
@@ -739,23 +739,28 @@ _cogl_journal_log_quad (float         x_1,
 
       cogl_get_modelview_matrix (&mv);
 
-      x = x_1, y = y_1, z = 0; w = 1;
+      x = position[X0], y = position[Y0], z = 0; w = 1;
       cogl_matrix_transform_point (&mv, &x, &y, &z, &w);
       v[0] = x; v[1] = y; v[2] = z;
       v += stride;
-      x = x_1, y = y_2, z = 0; w = 1;
+      x = position[X0], y = position[Y1], z = 0; w = 1;
       cogl_matrix_transform_point (&mv, &x, &y, &z, &w);
       v[0] = x; v[1] = y; v[2] = z;
       v += stride;
-      x = x_2, y = y_2, z = 0; w = 1;
+      x = position[X1], y = position[Y1], z = 0; w = 1;
       cogl_matrix_transform_point (&mv, &x, &y, &z, &w);
       v[0] = x; v[1] = y; v[2] = z;
       v += stride;
-      x = x_2, y = y_1, z = 0; w = 1;
+      x = position[X1], y = position[Y0], z = 0; w = 1;
       cogl_matrix_transform_point (&mv, &x, &y, &z, &w);
       v[0] = x; v[1] = y; v[2] = z;
     }
 
+#undef X0
+#undef Y0
+#undef X1
+#undef Y1
+
   for (i = 0; i < n_layers; i++)
     {
       /* XXX: See definition of GET_JOURNAL_VB_STRIDE_FOR_N_LAYERS for details
@@ -795,6 +800,11 @@ _cogl_journal_log_quad (float         x_1,
     COGL_MATERIAL_FLUSH_SKIP_GL_COLOR;
   entry->flush_options.fallback_layers = fallback_layers;
   entry->flush_options.disable_layers = disable_layers;
+  if (wrap_mode_overrides)
+    {
+      entry->flush_options.flags |= COGL_MATERIAL_FLUSH_WRAP_MODE_OVERRIDES;
+      entry->flush_options.wrap_mode_overrides = *wrap_mode_overrides;
+    }
   if (layer0_override_texture)
     {
       entry->flush_options.flags |= COGL_MATERIAL_FLUSH_LAYER0_OVERRIDE;
