@@ -16,9 +16,9 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * License along with this library. If not, see <http://www.gnu.org/licenses/>.
+ *
+ *
  */
 
 #if !defined(__COGL_H_INSIDE__) && !defined(CLUTTER_COMPILATION)
@@ -54,9 +54,25 @@ G_BEGIN_DECLS
 /**
  * cogl_path_fill:
  *
- * Fills the constructed shape using the current drawing color. The
- * current path is then cleared. To use the path again, call
- * cogl_path_fill_preserve() instead.
+ * Fills the interior of the constructed shape using the current
+ * drawing color. The current path is then cleared. To use the path
+ * again, call cogl_path_fill_preserve() instead.
+ *
+ * The interior of the shape is determined using the 'even-odd'
+ * rule. Any open sub-paths are treated as if there is an extra line
+ * joining the last point and first point. You can work out whether
+ * any point in the stage will be filled if you imagine drawing an
+ * infinitely long line in any direction from that point and then
+ * counting the number times it crosses a line in the path. If the
+ * number is odd it will be filled, otherwise it will not.
+ *
+ * See <xref linkend="fill-rule"/> for a demonstration of the fill
+ * rule.
+ *
+ * <figure id="fill-rule">
+ *   <title>Example of filling various paths</title>
+ *   <graphic fileref="fill-rule.png" format="PNG"/>
+ * </figure>
  **/
 void
 cogl_path_fill (void);
@@ -64,8 +80,10 @@ cogl_path_fill (void);
 /**
  * cogl_path_fill_preserve:
  *
- * Fills the constructed shape using the current drawing color and
- * preserves the path to be used again.
+ * Fills the interior of the constructed shape using the current
+ * drawing color and preserves the path to be used again. See
+ * cogl_path_fill() for a description what is considered the interior
+ * of the shape.
  *
  * Since: 1.0
  **/
@@ -161,13 +179,19 @@ cogl_path_rel_line_to (float x,
  * @center_y: Y coordinate of the elliptical arc center
  * @radius_x: X radius of the elliptical arc
  * @radius_y: Y radius of the elliptical arc
- * @angle_1: Angle in the unit-circle at which the arc begin
- * @angle_2: Angle in the unit-circle at which the arc ends
+ * @angle_1: Angle in degrees at which the arc begin
+ * @angle_2: Angle in degrees at which the arc ends
  *
  * Adds an elliptical arc segment to the current path. A straight line
  * segment will link the current pen location with the first vertex
  * of the arc. If you perform a move_to to the arcs start just before
  * drawing it you create a free standing arc.
+ *
+ * The angles are measured in degrees where 0° is in the direction of
+ * the positive X axis and 90° is in the direction of the positive Y
+ * axis. The angle of the arc begins at @angle_1 and heads towards
+ * @angle_2 (so if @angle_2 is less than @angle_1 it will decrease,
+ * otherwise it will increase).
  **/
 void
 cogl_path_arc (float center_x,
@@ -341,6 +365,52 @@ cogl_path_round_rectangle (float x_1,
                            float radius,
                            float arc_step);
 
+/**
+ * cogl_path_get:
+ *
+ * Gets a handle to the current path. The path can later be used again
+ * by calling cogl_path_set(). Note that the path isn't copied so if
+ * you later call any functions to add to the path it will affect the
+ * returned handle too. No reference is taken on the path so if you
+ * want to retain it you should take your own reference with
+ * cogl_handle_ref().
+ *
+ * Return value: a handle to the current path.
+ *
+ * Since: 1.4
+ */
+CoglHandle
+cogl_path_get (void);
+
+/**
+ * cogl_path_set:
+ * @handle: A %CoglHandle to a path
+ *
+ * Replaces the current path with @handle. A reference is taken on the
+ * handle so if you no longer need the path you should unref with
+ * cogl_handle_unref().
+ *
+ * Since: 1.4
+ */
+void
+cogl_path_set (CoglHandle handle);
+
+/**
+ * cogl_path_copy:
+ * @handle: A %CoglHandle to a path
+ *
+ * Returns a new copy of the path in @handle. The new path has a
+ * reference count of 1 so you should unref it with
+ * cogl_handle_unref() if you no longer need it.
+ *
+ * Internally the path will share the data until one of the paths is
+ * modified so copying paths should be relatively cheap.
+ *
+ * Return value: a copy of the path in @handle.
+ */
+CoglHandle
+cogl_path_copy (CoglHandle handle);
+
 G_END_DECLS
 
 #endif /* __COGL_PATH_H__ */
